package com.ruoyi.project.business.service.impl;

import cn.hutool.core.lang.tree.Tree;
import cn.hutool.core.lang.tree.TreeNodeConfig;
import cn.hutool.core.lang.tree.TreeUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.MessageUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.redis.RedisCache;
import com.ruoyi.project.business.domain.TbAgent;
import com.ruoyi.project.business.domain.TbArea;
import com.ruoyi.project.business.domain.TcLead;
import com.ruoyi.project.business.mapper.TbAgentMapper;
import com.ruoyi.project.business.mapper.TbAreaMapper;
import com.ruoyi.project.business.service.ITbAgentService;
import com.ruoyi.project.business.service.ITbAreaService;
import com.ruoyi.project.business.service.ITcAccountService;
import com.ruoyi.project.business.service.ITcLeadService;
import com.ruoyi.project.business.service.ITcProjectService;
import com.ruoyi.project.common.service.ICommonService;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.concurrent.CompletableFuture;

import static com.ruoyi.common.constant.CacheConstants.AGENT_KEY;
import static com.ruoyi.common.enums.SerialNumberPrefix.AGT;

/**
 * 【请填写功能名称】Service业务层处理
 *
 * @author ruoyi
 * @date 2024-04-16
 */
@Service
@RequiredArgsConstructor
public class TbAgentServiceImpl implements ITbAgentService {
    private final TbAgentMapper tbAgentMapper;

    private final ICommonService commonService;

    private final TbAreaMapper tbAreaMapper;

    private final ISysUserService sysUserService;

    private final RedisCache redisCache;

    private final ITbAreaService tbAreaService;

    private final ITcAccountService tcAccountService;
    
    private final ITcProjectService tcProjectService;
    
    private final ITcLeadService tcLeadService;
    
    
    

    /**
     * 查询【请填写功能名称】
     *
     * @param agentId 【请填写功能名称】主键
     * @return 【请填写功能名称】
     */
    @Override
    public TbAgent selectTbAgentByAgentId(Long agentId) {
        return tbAgentMapper.selectTbAgentByAgentId(agentId);
    }

    /**
     * 查询【请填写功能名称】列表
     *
     * @param tbAgent 【请填写功能名称】
     * @return 【请填写功能名称】
     */
    @Override
    public List<TbAgent> selectTbAgentList(TbAgent tbAgent) {
        return tbAgentMapper.selectTbAgentList(tbAgent);
    }

    /**
     * 新增【请填写功能名称】
     *
     * @param tbAgent 【请填写功能名称】
     * @return 结果
     */
    @Override
    @CacheEvict(value = AGENT_KEY, allEntries = true)
    public int insertTbAgent(TbAgent tbAgent) {
        String agentNo = redisCache.generateSerialNumber(AGT.getPrefix());
        tbAgent.setAgentNo(agentNo);

        repeatCheck(tbAgent);

        tbAgent.setCreateTime(DateUtils.getNowDate());
        tbAgent.setCreateBy(SecurityUtils.getUserId().toString());
        // 获取id
        if (tbAgent.getParentAgentId() == null) {
            tbAgent.setParentAgentId(0L);
        }
        // 非一级代理商时，也保存上级销售。如二级代理商的上级销售为一级代理商的上级销售   ： 20240513 from Tang by fangk
        if (tbAgent.getAgentLevel() != null && !tbAgent.getAgentType().equals("1")) {
            // TODO 注意这里只考虑两级代理商，多级代理商另需递归处理
            TbAgent parentAgent = tbAgentMapper.selectTbAgentByAgentId(tbAgent.getParentAgentId());
            if (parentAgent != null) {
                tbAgent.setParentSalerId(parentAgent.getParentSalerId());
            }
        }
        // id
        tbAgent.setAgentId(commonService.getCommonID("seq_area_agent"));
        // 数据权限
        tbAgent.setPermissionChar(handlePermissionChar(tbAgent));

        return tbAgentMapper.insertTbAgent(tbAgent);
    }

    private void repeatCheck(TbAgent tbAgent) {
        // 重复检查
        if (tbAgent.getAgentId() == null && StringUtils.isNotEmpty(tbAgent.getAgentNo())) {
            Integer exist = tbAgentMapper.existAgent(new TbAgent().setAgentNo(tbAgent.getAgentNo()));
            if (exist != null) {
                // 发番，不会发生
                throw new RuntimeException("代理商编号已存在");
            }
        }
        if (StringUtils.isNotEmpty(tbAgent.getAgentName())) {
            //Integer existName = tbAgentMapper.existAgent(new TbAgent().setAgentName(tbAgent.getAgentName()));
            Integer existName = tbAgentMapper.existAgent(tbAgent);
            if (existName != null) {
                throw new RuntimeException(MessageUtils.message("agent.name.repeat"));
            }
        }
    }


    /**
     * 拼接数据权限字符: 所属区域的数据权限字符+代理商的数据权限字符
     *
     * @param tbAgent 代理商
     */
    private String handlePermissionChar(TbAgent tbAgent) {
        TbArea tbArea = tbAreaMapper.selectTbAreaByAreaId(tbAgent.getAreaId());

        String permissionChar = "," + tbAgent.getAgentId();
        if (tbAgent.getParentAgentId() == 0L) {
            permissionChar += tbArea.getPermissionChar();
        } else {
            TbAgent parentAgent = tbAgentMapper.selectTbAgentByAgentId(tbAgent.getParentAgentId());
            permissionChar += parentAgent.getPermissionChar();
        }
        return permissionChar;
    }


    /**
     * 修改【请填写功能名称】
     *
     * @param tbAgent 【请填写功能名称】
     * @return 结果
     */
    @Override
    @CacheEvict(value = AGENT_KEY, allEntries = true)
    public int updateTbAgent(TbAgent tbAgent) {
        tbAgent.setUpdateTime(DateUtils.getNowDate());
        // 获取id
        if (tbAgent.getParentAgentId() == null) {
            tbAgent.setParentAgentId(0L);
        }
        this.repeatCheck(tbAgent);
        tbAgent.setPermissionChar(handlePermissionChar(tbAgent));
        return tbAgentMapper.updateTbAgent(tbAgent);
    }

    /**
     * 批量删除【请填写功能名称】
     *
     * @param agentIds 需要删除的【请填写功能名称】主键
     * @return 结果
     */
    @Override
    @CacheEvict(value = AGENT_KEY, allEntries = true)
    public int deleteTbAgentByAgentIds(Long[] agentIds) {
        preCheckDelete(agentIds);

        return tbAgentMapper.deleteTbAgentByAgentIds(agentIds);
    }

    /**
     * 代理商删除前的校验
     * @param agentIds
     */
    private void preCheckDelete(Long[] agentIds) {

        // 检查是否存在下级代理商
        CompletableFuture<Integer> existNextAgentTask = CompletableFuture.supplyAsync(() -> tbAgentMapper.existNextAgent(agentIds));
        // 检查是否存在关联的用户
        CompletableFuture<Integer> existUserTask = CompletableFuture.supplyAsync(() -> sysUserService.existUserByAgentIds(agentIds));
        // 检查是否存在关联的客户
        CompletableFuture<Integer> existCustomerTask = CompletableFuture.supplyAsync(() -> tcAccountService.existCustomerByAgentIds(agentIds));
        // 检查是否存在关联的项目
        CompletableFuture<Integer> existProjectTask = CompletableFuture.supplyAsync(() -> tcProjectService.existProjectByAgentIds(agentIds));
        // 检查是否存在关联的线索
        CompletableFuture<Boolean> existLeadTask = CompletableFuture.supplyAsync(() -> tcLeadService.getBaseMapper().exists(Wrappers.lambdaQuery(TcLead.class).in(TcLead::getOwnerAgent, Arrays.asList(agentIds))));
        
        CompletableFuture.allOf(existNextAgentTask, existCustomerTask, existUserTask, existProjectTask, existLeadTask).join();

        if (existNextAgentTask.join() != null) {
            throw new RuntimeException(MessageUtils.message("agent.delFail.existSubAgent"));
        }
        if (existUserTask.join() != null) {
            throw new RuntimeException(MessageUtils.message("agent.delFail.existUser"));
        }
        if (existCustomerTask.join() != null) {
            throw new RuntimeException(MessageUtils.message("agent.delFail.existCustomer"));
        }
        if (existProjectTask.join() != null) {
            throw new RuntimeException(MessageUtils.message("agent.delFail.existProject"));
        }
        if (existLeadTask.join()) {
            throw new RuntimeException(MessageUtils.message("agent.delFail.existLead"));
        }
        

    }

    /**
     * 删除【请填写功能名称】信息
     *
     * @param agentId 【请填写功能名称】主键
     * @return 结果
     */
    @Override
    public int deleteTbAgentByAgentId(Long agentId) {
        preCheckDelete(new Long[]{agentId});
        return tbAgentMapper.deleteTbAgentByAgentId(agentId);
    }

    @Override
    public List<Tree<String>> selectAgentTreeList(TbAgent agent) {
        List<TbAgent> agents = tbAgentMapper.selectTbAgentList(agent);

        // 如果不存在总部根节点，手动添加
        boolean existRoot = agents.stream().anyMatch(item -> item.getAgentId() == 0L);
        if (!existRoot) {
            // 添加根节点：手动添加id：0；parent_id：-1作为总部（根节点）
            TbAgent root = new TbAgent();
            root.setAgentId(0L);
            root.setAgentName("总部");
            root.setParentAgentId(-1L);
            agents.add(root);
        }

        //配置 
        TreeNodeConfig config = new TreeNodeConfig();
        // 最大递归深度 不设置表示不限制深度
        config.setDeep(10);

        List<Tree<String>> list = TreeUtil.build(agents, "-1", config, (node, tree) -> {
            tree.setId(node.getAgentId().toString());
            tree.setName(node.getAgentName());
            tree.setParentId(node.getParentAgentId() == null ? "-1" : node.getParentAgentId().toString());

            // 额外的值/或者使用默认的name，在前端自定义label
            tree.put("label", node.getAgentName());
        });

        return list;
    }

    @Override
    public List<TbAgent> selectAgentByAreaId(Long areaId, Long agentId) {
        // 查询区域下的一级代理商
        TbAgent agent = new TbAgent();
        agent.setAreaId(areaId);
        //agent.setParentAgentId(0L);
        agent.setAgentType("1");

        List<TbAgent> tbAgents = tbAgentMapper.selectTbAgentList(agent);
        // 排除自己
        tbAgents.removeIf(item -> item.getAgentId().equals(agentId));
        return tbAgents;
    }


    @Override
    public List<SysUser> selectSaleByAreaId(Long areaId) {
        // 所属区域的销售经理角色的user
        //SysRole
        List<SysUser> users = sysUserService.selectUserByRoleKeyAndAreaId("sales_manager", areaId);
        return users;
    }

    @Override
    @CacheEvict(value = AGENT_KEY, allEntries = true)
    public int changeStatus(TbAgent agent) {
        return tbAgentMapper.updateTbAgent(agent);
    }

    /**
     * 获取客户全部代理商
     */
    @Override
    public List<TbAgent> selectForAccount() {
        return tbAgentMapper.selectForAccount(SecurityUtils.getPermissionCharSea());
    }

    @Override
    public List<TbAgent> selectForProject() {
        return tbAgentMapper.selectForProject(SecurityUtils.getPermissionCharSea());
    }

    @Override
    @Cacheable(value = AGENT_KEY, key = "#agentId + '_' + #justSubs + '_' + #includeSelf", unless = "#result == null")
    public List<TbAgent> selectChildAreaList(Long agentId, boolean justSubs, boolean includeSelf) {
        TbAgent tbAgent = tbAgentMapper.selectTbAgentByAgentId(agentId);
        if (tbAgent == null) {
            throw new RuntimeException(MessageUtils.message("agent.not.exist"));
        }
        
        List<TbAgent> tbAgents;
        if (justSubs) {
            tbAgents = tbAgentMapper.selectTbAgentList(new TbAgent().setParentAgentId(agentId));
            if (includeSelf) {
                tbAgents.add(tbAgent);
            }
        } else {
            // 反转字符串
            String permission = new StringBuilder(tbAgent.getPermissionChar()).reverse().toString();
            //permission += justSubs ? "_," : "%";
            tbAgents = tbAgentMapper.selectChildAgentList(permission);
            if (!includeSelf) {
                tbAgents.removeIf(area -> area.getAgentId().equals(agentId));
            }
        }
       
        return tbAgents;
    }

    @Override
    public List<TbAgent> selectAgentListByOwner(Long ownerId) {
        Long areaId = sysUserService.selectUserById(ownerId).getAreaId();
        List<Long> areaIds = tbAreaService.selectChildAreaIdsByAreaId(areaId);
        return tbAgentMapper.selectAgentListByAreaIds(areaIds);
    }
}
