package com.ruoyi.project.business.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.URLUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ruoyi.common.enums.FileBuzType;
import com.ruoyi.common.importExcel.excel.helper.BasicTableInfoHelper;
import com.ruoyi.common.importExcel.excel.tempInfo.LeadInfoHelper;
import com.ruoyi.common.importExcel.utils.ExcelUtils;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.MessageUtils;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.common.utils.SpecialCharacterEscaper;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.redis.RedisCache;
import com.ruoyi.framework.web.domain.BaseEntity;
import com.ruoyi.project.business.domain.TbArea;
import com.ruoyi.project.business.domain.TbCountry;
import com.ruoyi.project.business.domain.TbFile;
import com.ruoyi.project.business.domain.TcLead;
import com.ruoyi.project.business.domain.TcLeadForRelated;
import com.ruoyi.project.business.domain.TcLeadHistory;
import com.ruoyi.project.business.domain.vo.board.BarChartVo;
import com.ruoyi.project.business.domain.vo.country.AreaWithCountriesVo;
import com.ruoyi.project.business.domain.vo.lead.*;
import com.ruoyi.project.business.enums.LeadStatusEnum;
import com.ruoyi.project.business.enums.MessageTypeEnum;
import com.ruoyi.project.business.mapper.TcLeadMapper;
import com.ruoyi.project.business.service.ITbAreaService;
import com.ruoyi.project.business.service.ITbFileService;
import com.ruoyi.project.business.service.ITcLeadHistoryService;
import com.ruoyi.project.business.service.ITcLeadService;
import com.ruoyi.project.common.domain.Message;
import com.ruoyi.project.common.service.ISseEmitterService;
import com.ruoyi.project.system.domain.SysDictData;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.mapper.SysDictDataMapper;
import com.ruoyi.project.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.BeanUtils;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.ZoneId;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.ruoyi.common.constant.Constants.*;
import static com.ruoyi.common.enums.SerialNumberPrefix.LEAF;
import static com.ruoyi.project.business.enums.LeadStatusEnum.*;

/**
 * 线索管理Service业务层处理
 *
 * @author ruoyi
 * @date 2024-04-10
 */
@Service
@RequiredArgsConstructor
public class TcLeadServiceImpl extends ServiceImpl<TcLeadMapper, TcLead> implements ITcLeadService {
    // 可分配状态: 待分配、 已分配、已回收
    private static final Set<String> ASSIGNABLE_STATUS = new HashSet<>(Arrays.asList("1", "2", "3"));

    // private static final Map<String, Function<TbArea, String>> AREA_FUNCTION_MAP = new HashMap<String, Function<TbArea, String>>() {{
    //    put(US, TbArea::getAreaNameUS);
    //    put(RU, TbArea::getAreaNameRU);
    //    put(ES, TbArea::getAreaNameES);
    //    put(CN, TbArea::getAreaName);
    //}};

    private final ITbFileService tbFileService;

    private final RedisCache redisCache;

    private final ISseEmitterService sseEmitterService;

    private final ISysUserService sysUserService;

    private final ITcLeadHistoryService tcLeadHistoryService;

    private final SysDictDataMapper sysDictDataMapper;

    private final ITbAreaService tbAreaService;

    private final ThreadPoolTaskExecutor threadPoolTaskExecutor;

    /**
     * 查询线索管理
     *
     * @param leadId 线索管理主键
     * @return 线索管理
     */
    @Override
    public LeadRes selectTcLeadByLeadId(Long leadId) {
        LeadRes tcLead = baseMapper.selectTcLeadByLeadId(leadId);
        // TcLead tcLead = getById(leadId);

        if (null == tcLead) {
            throw new RuntimeException(MessageUtils.message("lead.error.notexistOrDel"));
        }

        // 失效原因查询
        if (INVALID.getDesc().equals(tcLead.getStatus())) {
            TcLeadHistory one = tcLeadHistoryService.getOne(Wrappers.lambdaQuery(TcLeadHistory.class)
                    .eq(TcLeadHistory::getLeadId, leadId)
                    // TODO 枚举
                    .eq(TcLeadHistory::getActionType, 2)
                    .eq(TcLeadHistory::getRecoveryType, 99)
                    .orderByDesc(TcLeadHistory::getCreateTime)
                    .last("limit 1"));
            if (one != null) {
                tcLead.setInvalidReason(one.getRecoveryReason());
                // tcLead.setLostRemark(one.getRecoveryReason());
            }
        }

        LeadRes leadRes = new LeadRes();
        BeanUtils.copyProperties(tcLead, leadRes);
        if (0L == leadRes.getOwnerAgent()) {
            leadRes.setOwnerAgentName(MessageUtils.message("agent.headquarters"));
        }

        // 线索时间不要了
        // leadRes.setLeadTime(DateUtils.LocalTimeDate(tcLead.getLeadTime()));
        // 查询区域
        // if (tcLead.getAreaId() != null) {
        //    tcLead.setAreaName(tbAreaMapper.selectTbAreaByAreaId(tcLead.getAreaId()).getAreaName());
        //}

        // 查询附件
        List<TbFile> tbFiles = tbFileService.selectTbFileListByBuzIdAndBuzType(leadId, FileBuzType.LEAD.getType());
        String urlStr = tbFiles.stream().map(TbFile::getFilePath).collect(Collectors.joining(","));
        leadRes.setTbFiles(tbFiles);
        leadRes.setAnnex(urlStr);
        return leadRes;
    }

    /**
     * 查询关联线索
     *
     * @return 线索管理
     */
    @Override
    public List<TcLeadForRelated> selectLeadListByLoginId() {
        Long loginId = SecurityUtils.getUserId();
        return baseMapper.selectLeadListByLoginId(loginId);
    }

    public List<TcLeadForRelated> selectLeadListByLeadId(LeadRelatedQueryReq tcLead) {
        return baseMapper.selectLeadListByLeadId(tcLead);
    }

    public List<TcLeadForRelated> selectLeadListByQuery(LeadRelatedQueryReq tcLead) {
        Long loginId = SecurityUtils.getUserId();
        SpecialCharacterEscaper.escapeSpecialCharacters(tcLead, LeadRelatedQueryReq::getQuery);
        tcLead.setLoginId(loginId);
        return baseMapper.selectLeadListByQuery(tcLead);
    }

    /**
     * 查询线索管理列表
     *
     * @param tcLead 线索管理
     * @return 线索管理
     */
    @Override
    public List<LeadRes> selectTcLeadList(LeadListReq tcLead) {
        SpecialCharacterEscaper.escapeSpecialCharacters(tcLead, LeadListReq::getLeadNo, LeadListReq::getAccountNo,
                LeadListReq::getCompany, LeadListReq::getOwnerName, LeadListReq::getContact, LeadListReq::getRequirement,
                LeadListReq::getAddress, LeadListReq::getCreateByNickName);

        tcLead.setPermissionChar(SecurityUtils.getPermissionCharSea());
        Long userId = SecurityUtils.getUserId();
        Assert.notNull(userId, "用户未登录");
        // 代理商销售经理的场合
        if (SecurityUtils.hasRole("agent_sales")) {
            tcLead.setLoginUserId(userId);
        }

        // List<TbArea> allArea = tbAreaMapper.selectTbAreaList(new AreaParam());
        // Map<Long, TbArea> areaId2AreaMap = allArea.stream().collect(Collectors.toMap(TbArea::getAreaId, Function.identity()));
        String lan = LocaleContextHolder.getLocale().getCountry();
        // TODO 但是使用这种方式显著降低了导出速度，需要优化 关于 区域和国家 数据可以内存化，使用 cacheManager
        // TODO 检查一下这里查询的 history 子集的查询是否有必要：替换成取最新的一个
        tcLead.setLan(lan);

        List<LeadRes> tcLeads = baseMapper.selectTcLeadList(tcLead);
        String headquarters = MessageUtils.message("agent.headquarters");
        for (LeadRes lead : tcLeads) {
            // 是否可分配： 1.创建人且未分配、已分配【代理商直接分配给自己】(创建人不是owner时，说明线索已经分配，此时自己不可再分配) 2.（其他人）已分配给自己且状态合法
            // 负责人是自己且状态是待分配、已分配、已回收
            lead.setAssignable(Objects.equals(lead.getOwner(), userId) && ASSIGNABLE_STATUS.contains(lead.getStatus()) || lead.getOwner() == null && Arrays.asList("1", "3").contains(lead.getStatus()));
            Optional.ofNullable(lead.getTcLeadHistories()).flatMap(histories -> histories.stream().max(Comparator.comparing(BaseEntity::getCreateTime))).ifPresent(history -> {
                // 是否可回收： 最后一次操作是由本人分配的 && 操作类型是分配 && 状态是已分配        // 1L == 比较可能会空指针
                lead.setRecyclable(userId.equals(history.getOriUser()) && "2".equals(lead.getStatus()) && Long.valueOf(1).equals(history.getActionType()));
            });
            // 是否可跟进/可编辑 1.负责人是自己 2.状态非无效
            lead.setFollowable(Objects.equals(lead.getOwner(), userId) && !"8".equals(lead.getStatus()));
            // 是否可删除: 负责人 + 状态是待分配 或者 市场部创建的线索（市场部owner 是空，创建人是自己）
            lead.setDeletable("1".equals(lead.getStatus()) && (Objects.equals(lead.getOwner(), userId) || (null == lead.getOwner() && userId.equals(lead.getCreateBy()))));
            // 是否可转化: 负责人是自己 + 状态是跟进中
            lead.setConvertable(Objects.equals(lead.getOwner(), userId) && "4".equals(lead.getStatus()));

            // 是否可转化项目： 负责人是自己 + 状态是转化客户
            lead.setConvertProjectable(Objects.equals(lead.getOwner(), userId) && "5".equals(lead.getStatus()));
            
            // 所属代理商总部赋值
            if (0L == lead.getOwnerAgent()) {
                lead.setOwnerAgentName(headquarters);
            }

            // 回显时间 地区化 // 暂时取消，注意新增、修改；
            // lead.setLeadTime(DateUtils.LocalTimeDate(lead.getLeadTime()));

            // TbArea tbArea = areaId2AreaMap.get(lead.getAreaId());
            // if (tbArea != null) {
            //    lead.setAreaName(AREA_FUNCTION_MAP.getOrDefault(lan, TbArea::getAreaName).apply(tbArea));
            //}

        }
        return tcLeads;
    }

    /**
     * 新增线索管理
     *
     * @param lead 线索管理
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean insertTcLead(LeadFormReq lead) {
        // TODO 将这个状态改成枚举
        // 线索编号：
        String leadNo = redisCache.generateSerialNumber(LEAF.getPrefix());
        lead.setLeadNo(leadNo);
        lead.setStatus(LeadStatusEnum.TO_BE_ALLOCATED.getCode());
        lead.setPermissionChar(SecurityUtils.getPermissionCharIns());

        // Optional.ofNullable(lead.getLeadTime()).ifPresent(leadTime -> {
        //    lead.setLeadTime(DateUtils.CNTimeDate(lead.getLeadTime()));
        //});

        // 如果创建线索用户是区域:全球，所属：总部；即表示是市场部创建的线索，负责人是空。 其他情况，负责人：自己
        if (SecurityUtils.getDeptId() != 0L || SecurityUtils.getLoginUser().getAreaId() != 0L) {
            lead.setOwner(SecurityUtils.getUserId());
            // 如果是代理商创建的线索，所属代理商是自己，并且分配给自己
            if (SecurityUtils.getDeptId() != 0L) {
                lead.setOwnerAgent(SecurityUtils.getDeptId());
                lead.setStatus("2");
            }
        }

        // int rows = leadMapper.insertTcLead(lead);
        boolean r1 = save(lead);
        // 保存附件
        saveLeadFile(lead);
        if (!r1) {
            throw new RuntimeException("保存线索失败");
        }
        return r1;
    }

    /**
     * 修改线索管理
     *
     * @param tcLead 线索管理
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean updateTcLead(LeadFormReq tcLead) {
        // 删除附件 url
        if (CollUtil.isNotEmpty(tcLead.getTbFiles())) {
            tbFileService.deleteTbFileByIds(tcLead.getTbFiles().stream().map(TbFile::getId).toArray(Long[]::new));
        }

        // Optional.ofNullable(tcLead.getLeadTime()).ifPresent(leadTime -> {
        //     tcLead.setLeadTime(DateUtils.CNTimeDate(leadTime, SecurityUtils.getTimeZone()));
        // });
        // 保存附件
        saveLeadFile(tcLead);
        tcLead.setUpdateTime(new Date());
        tcLead.setUpdateBy(SecurityUtils.getUserId().toString());
        // return baseMapper.updateTcLead(tcLead);
        return updateById(tcLead);
    }

    private void saveLeadFile(LeadFormReq lead) {
        // 保存附件
        String annex = lead.getAnnex();
        if (StringUtils.isNotBlank(annex)) {
            for (String fileUrl : annex.split(",")) {
                // 获取文件名：
                // 处理空格
                fileUrl = fileUrl.replaceAll(" ", "%20");
                // 解析URL
                String path = URLUtil.getPath(fileUrl);
                // 从路径中获取文件名
                String fileName = FileUtil.getName(path);
                TbFile tbFile = new TbFile(lead.getLeadId(), FileBuzType.LEAD.getType(), fileName, fileName, fileUrl);
                tbFile.setCreateBy(SecurityUtils.getUserId().toString());
                tbFile.setCreateTime(new Date());
                tbFileService.insertTbFile(tbFile);
            }
        }
    }
//
//    /**
//     * 批量删除线索管理
//     *
//     * @param leadIds 需要删除的线索管理主键
//     * @return 结果
//     */
//    @Override
//    public int deleteTcLeadByLeadIds(Long[] leadIds) {
//        return baseMapper.deleteTcLeadByLeadIds(leadIds);
//    }
//
//    /**
//     * 删除线索管理信息
//     *
//     * @param leadId 线索管理主键
//     * @return 结果
//     */
//    @Override
//    public int deleteTcLeadByLeadId(Long leadId) {
//        return baseMapper.deleteTcLeadByLeadId(leadId);
//    }

    /**
     * 查询客户相关线索
     *
     * @param tcLead 线索
     * @return 结果
     */
    @Override
    public List<TcLeadForRelated> selectLeadListByAccount(LeadRelatedQueryReq tcLead) {
        return baseMapper.selectLeadListByAccount(tcLead);
    }

    /**
     * 回收
     *
     * @param recovery 线索
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean recovery(RecoveryReq recovery) {
        Long userId = SecurityUtils.getUserId();
        Long msgTo = recovery.getOwner();
        TcLead tcLead = new TcLead();
        tcLead.setLeadId(recovery.getLeadId());
        // 如果原因是失效，更新线索状态; // 99:失效; // 8:回收
        if ("99".equals(recovery.getRecoveryType())) {
            tcLead.setStatus(INVALID.getCode());
            tcLead.setUnvalidReason(recovery.getRecoveryReason());
        } else {
            tcLead.setOwner(userId);
            // 根据区域和代理商判断
            if (SecurityUtils.getDeptId() != 0L) {
                // 代理商：ownerAgent: 设为自己
                tcLead.setOwnerAgent(SecurityUtils.getDeptId());
            }
            // 回收状态
            tcLead.setStatus(RECYCLED.getCode());
        }
        tcLead.setPermissionChar(SecurityUtils.getPermissionCharIns());

        // tcLead.setUpdateBy(userId.toString());
        // tcLead.setUpdateTime(new Date());
        // int r1 = baseMapper.updateTcLead(tcLead);
        boolean r1 = updateById(tcLead);

        // 履历
        TcLeadHistory history = BeanUtil.copyProperties(recovery, TcLeadHistory.class);
        history.setOriUser(userId);
        // history.setCreateBy(userId.toString());
        // history.setCreateTime(new Date());
        // int r2 = tcLeadHistoryMapper.insertTcLeadHistory(history);

        boolean r2 = tcLeadHistoryService.save(history);

        Message message = new Message();
        // message.setMessageContent("您有一条线索被回收，请注意查看。线索编号：" + recovery.getLeadNo());
        // sseEmitterService.sendSaveMessage(Collections.singletonList(msgTo), message);
        Map<String, String> allMessage = MessageUtils.allMessage("lead.message.recovered");
        message.setMessageType(MessageTypeEnum.LEAD.getValue());
        message.setMessageContent(allMessage.get(CN) + recovery.getLeadNo());
        message.setMessageContentUS(allMessage.get(US) + recovery.getLeadNo());
        message.setMessageContentRU(allMessage.get(RU) + recovery.getLeadNo());
        message.setMessageContentES(allMessage.get(ES) + recovery.getLeadNo());
        message.setCreateBy(userId.toString());
        sseEmitterService.sendSaveMessage(Collections.singletonList(msgTo), message);
        // return r1 + r2;
        return r1 && r2;
    }

    @Override
    public List<TcLead> selectLeadsFromMarketAndUnassigned() {
        return baseMapper.selectLeadsFromMarketAndUnassigned();
    }


    /**
     * TODO 这里有大事务问题，需要处理
     *
     * @param allots 分配线索
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean allot(List<AllotReq> allots) {
        if (CollectionUtils.isEmpty(allots)) {
            throw new RuntimeException("分配线索信息不能为空");
        }

        Long userId = SecurityUtils.getUserId();

        // 校验并根据返回负责人信息
        final SysUser sysUser = checkAndHandleOwner(allots, userId);

        Date now = new Date();

        List<TcLead> tcLeads = new ArrayList<>();
        List<TcLeadHistory> histories = new ArrayList<>();

        for (AllotReq allot : allots) {
            TcLead tLead = BeanUtil.copyProperties(allot, TcLead.class);
            tLead.setUpdateBy(userId.toString());
            tLead.setUpdateTime(now);

            // 代理商用户设置所属代理商
            if (sysUser.getDeptId() != 0) {
                tLead.setOwnerAgent(sysUser.getDeptId());
                tLead.setPermissionChar(sysUser.getDeptId().toString());
            } else {
                tLead.setPermissionChar(sysUser.getAreaId().toString());
            }
            tcLeads.add(tLead);

            // r1 += baseMapper.updateTcLead(tLead);
            TcLeadHistory history = new TcLeadHistory(allot.getLeadId(), userId, 1L, null, null, allot.getOwner(), userId, now, null);
            TcLeadHistory tHistory = BeanUtil.copyProperties(history, TcLeadHistory.class);
            histories.add(tHistory);

        }

        boolean r1 = updateBatchById(tcLeads);
        boolean r2 = tcLeadHistoryService.saveBatch(histories);

        Message message = new Message();
        // message.setMessageContent("您有一条线索待处理，请注意查看。线索编号：" + recovery.getLeadNo());
        Map<String, String> allMessage = MessageUtils.allMessage("lead.message.allot");
        message.setMessageType(MessageTypeEnum.LEAD.getValue());
        String leadNos = allots.stream().map(AllotReq::getLeadNo).collect(Collectors.joining(","));
        message.setMessageContent(allMessage.get(CN) + leadNos);
        message.setMessageContentUS(allMessage.get(US) + leadNos);
        message.setMessageContentRU(allMessage.get(RU) + leadNos);
        message.setMessageContentES(allMessage.get(ES) + leadNos);
        message.setCreateBy(userId.toString());
        sseEmitterService.sendSaveMessage(Collections.singletonList(allots.get(0).getOwner()), message);

        return r1 && r2;
    }

    /**
     * 检查并处理
     * TODO 批量分配还没处理
     */
    private SysUser checkAndHandleOwner(List<AllotReq> allots, Long userId) {

        if (CollUtil.isNotEmpty(allots)) {
            for (AllotReq allot : allots) {
                // TcLead curLead = baseMapper.selectTcLeadByLeadId(allot.getLeadId());
                TcLead curLead = getById(allot.getLeadId());
                // TODO i18n
                if (curLead == null) {
                    throw new RuntimeException("线索不存在，线索id：【" + allot.getLeadId() + "】");
                } else if (!userId.toString().equals(curLead.getCreateBy()) && !userId.equals(curLead.getOwner())) {
                    // 只有创建人或者owner才能分配
                    throw new RuntimeException("只有创建人或者owner才能分配，线索编号：" + curLead.getLeadNo());
                } else if (!ASSIGNABLE_STATUS.contains(curLead.getStatus())) {
                    // 只有未分配和已分配才能分配
                    throw new RuntimeException("只有未分配、已分配和已回收才能分配, 线索编号：" + curLead.getLeadNo());
                }

            }
        } else {
            throw new RuntimeException("线索ID不能为空");
        }

        // 分配到对方的权限字符
        SysUser sysUser = sysUserService.selectUserById(allots.get(0).getOwner());
        if (sysUser == null) {
            throw new RuntimeException("分配用户不存在");
        }

        return sysUser;
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean convertCustomer(ConvertReq convertReq) {
        TcLead tcLead = BeanUtil.copyProperties(convertReq, TcLead.class);
        tcLead.setUpdateBy(SecurityUtils.getUserId().toString());
        tcLead.setUpdateTime(new Date());
        // int r1 = baseMapper.updateTcLead(tcLead);
        boolean r1 = updateById(tcLead);

        // 履历 如果已经转化客户，不再记录
        boolean exists = tcLeadHistoryService.getBaseMapper().exists(Wrappers.<TcLeadHistory>lambdaQuery().eq(TcLeadHistory::getLeadId, convertReq.getLeadId()).eq(TcLeadHistory::getActionType, 3L));
        if (exists) {
            return r1;
        }
        TcLeadHistory history = new TcLeadHistory(convertReq.getLeadId(), SecurityUtils.getUserId(), 3L, null, null, convertReq.getRelativeAccount(), SecurityUtils.getUserId(), new Date(), convertReq.getRemark());
        // int r2 = tcLeadHistoryMapper.insertTcLeadHistory(history);
        boolean r2 = tcLeadHistoryService.save(history);
        // return r1 + r2;
        return r1 && r2;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean convertProject(Long leadId, Long relativeProject) {
        Assert.notNull(leadId, MessageUtils.message("lead.convert.validLeadId"));
        Assert.notNull(relativeProject, MessageUtils.message("lead.convert.validProjectId"));

        TcLead byId = getById(leadId);
        if (byId == null) {
            throw new RuntimeException(MessageUtils.message("lead.convert.notExistLead"));
        } else if (!CONVERTED_CUSTOMER.getCode().equals(byId.getStatus())) {
            // 只有转化客户才能转化项目
            throw new RuntimeException(MessageUtils.message("lead.convert.mustBeConvertCust"));
        }

        TcLead tcLead = new TcLead();
        tcLead.setStatus(CONVERTED_PROJECT.getCode());
        tcLead.setLeadId(leadId);
        tcLead.setRelativeProject(relativeProject);

        boolean r1 = updateById(tcLead);

        // 履历 如果已经转化项目，不再记录
        boolean exists = tcLeadHistoryService.getBaseMapper().exists(Wrappers.<TcLeadHistory>lambdaQuery().eq(TcLeadHistory::getLeadId, leadId).eq(TcLeadHistory::getActionType, 4L));
        if (exists) {
            return r1;
        }
        TcLeadHistory history = new TcLeadHistory().setLeadId(leadId).setActionType(4L).setOriUser(SecurityUtils.getUserId()).setGetUser(relativeProject);
        boolean r2 = tcLeadHistoryService.save(history);
        return r1 && r2;
    }


    /**
     * 客户统计
     * 下面是sql版本
     * <p>
     * SELECT DATE_FORMAT(lead_time, '%Y-%m') AS month,
     * COUNT(*)                        AS lead_count
     * FROM tc_lead
     * WHERE lead_time BETWEEN DATE_SUB(NOW(), INTERVAL 6 MONTH) AND NOW()
     * GROUP BY DATE_FORMAT(lead_time, '%Y-%m');
     *
     * @return
     */
    @Override
    public BarChartVo getLeadCountStat(LocalDate startDate, LocalDate endDate, String statMode) {
        ZoneId zoneFrom = ZoneId.of(SecurityUtils.getTimeZone());
        startDate = startDate.atStartOfDay(zoneFrom).withZoneSameInstant(ZoneId.systemDefault()).toLocalDate();
        endDate = endDate.atStartOfDay(zoneFrom).withZoneSameInstant(ZoneId.systemDefault()).toLocalDate();


        Function<LocalDate, LocalDate> fun;
        String pattern;

        switch (statMode) {
            case "day":
                fun = date -> date.plusDays(1);
                pattern = "yyyy-MM-dd";
                break;
            case "month":
                fun = date -> date.plusMonths(1);
                pattern = "yyyy-MM";
                break;
            case "year":
                fun = date -> date.plusYears(1);
                pattern = "yyyy";
                break;
            default:
                // TODO i18n
                throw new RuntimeException("统计模式错误");
        }
        endDate = fun.apply(endDate);


        // 查询数据 // TODO 这里暂时使用 stream 处理了，因为 crm 系统数据量不会太大。如果后续耗时过长使用本方法 javadoc 中的 sql修改
        List<TcLead> tcLeads = baseMapper.selectLeadCountStat(startDate, endDate, SecurityUtils.getPermissionCharSea());
        SimpleDateFormat sdf = new SimpleDateFormat(pattern);
        Map<String, Long> collect = tcLeads.stream().filter(lead -> lead.getCreateTime() != null).collect(Collectors.groupingBy(lead -> sdf.format(lead.getCreateTime()), Collectors.counting()));
        // 使用有序map
        collect = new TreeMap<>(collect);

        // 补全数据; 按照年月日处理
        while (startDate.isBefore(endDate)) {
            String key = sdf.format(DateUtil.date(startDate));
            collect.putIfAbsent(key, 0L);
            startDate = fun.apply(startDate);
        }

        return new BarChartVo(collect.keySet(), collect.values());

    }

    @Override
    public List<LeadRes> curAgent() {
        return baseMapper.curAgent(SecurityUtils.getPermissionCharSea());
    }


    // TODO 这个方法代码结构需要重构
    @Override
    public ExcelUtils.UploadResult<LeadInfoHelper> checkLeadImport(BasicTableInfoHelper basicTableInfoHelper, List<LeadInfoHelper> leadInfoHelpers) {

        // fast fail
        if (CollectionUtils.isEmpty(leadInfoHelpers)) {
            return ExcelUtils.checkData(leadInfoHelpers, it -> new ArrayList<>());
        }


        // 获取当前i18n
        Locale locale = LocaleContextHolder.getLocale();
        Function<SysDictData, String> dictGetter;
        Function<TbArea, String> areaGetter;
        Function<TbCountry, String> countryGetter;

        switch (locale.getCountry()) {
            case US:
                dictGetter = SysDictData::getDictLabelUS;
                areaGetter = TbArea::getAreaNameUS;
                countryGetter = TbCountry::getCountryNameUS;
                break;
            case RU:
                dictGetter = SysDictData::getDictLabelRU;
                areaGetter = TbArea::getAreaNameRU;
                countryGetter = TbCountry::getCountryNameRU;
                break;
            case ES:
                dictGetter = SysDictData::getDictLabelES;
                areaGetter = TbArea::getAreaNameES;
                countryGetter = TbCountry::getCountryNameES;
                break;
            default:
                dictGetter = SysDictData::getDictLabel;
                areaGetter = TbArea::getAreaName;
                countryGetter = TbCountry::getCountryName;
        }

        // TODO 收集成 map 集合
        // TODO cache 2 redis?
        // 来源
        CompletableFuture<Map<String, SysDictData>> originTask = asyncLoadDictFuture(dictGetter, "lead_source");
        
        // 行业
        // CompletableFuture<Map<String, SysDictData>> sectorTask = asyncLoadDictFuture(dictGetter, "lead_industry");

        // 优先级
        // CompletableFuture<Map<String, SysDictData>> priorityTask = CompletableFuture.supplyAsync(() -> getLocalLabel2DictMap(dictGetter, "lead_priority"), threadPoolTaskExecutor);
        CompletableFuture<Map<String, SysDictData>> priorityTask = asyncLoadDictFuture(dictGetter, "lead_priority");

        // 类型
        CompletableFuture<Map<String, SysDictData>> typeTask = asyncLoadDictFuture(dictGetter, "account_type");

        // 公司类型
        CompletableFuture<Map<String, SysDictData>> companyTypeTask = asyncLoadDictFuture(dictGetter, "company_type");

        // 项目类型
        CompletableFuture<Map<String, SysDictData>> projectTypeTask = asyncLoadDictFuture(dictGetter, "project_type");
        
        // 需求类型
        CompletableFuture<Map<String, SysDictData>> requirementTask = asyncLoadDictFuture(dictGetter, "lead_requirements");
        
        // 产品意向
        CompletableFuture<Map<String, SysDictData>> productIntentionTask = asyncLoadDictFuture(dictGetter, "intentional_product");

        //  区域 + 国家
        CompletableFuture<Map<String, AreaWithCountriesVo>> areaWithCountriesMap = CompletableFuture.supplyAsync(() -> {
            List<AreaWithCountriesVo> tbCountries = tbAreaService.selectTbAreaWithCountriesList(new TbArea());
            return tbCountries.stream().collect(Collectors.toMap(areaGetter, area -> area));
        }, threadPoolTaskExecutor);

        // 汇总任务，并且处理异常
        CompletableFuture<Void> allOf = CompletableFuture.allOf(originTask, priorityTask, typeTask,
                companyTypeTask, projectTypeTask, requirementTask, productIntentionTask,
                areaWithCountriesMap).exceptionally(e -> {
            throw new RuntimeException("字典数据异常", e);
        });

        allOf.join();

        Map<String, SysDictData> originMap = originTask.join();
        // Map<String, SysDictData> sectorMap = sectorTask.join();
        Map<String, SysDictData> priorityMap = priorityTask.join();
        Map<String, SysDictData> typeMap = typeTask.join();
        Map<String, AreaWithCountriesVo> areaWithCountriesVoMap = areaWithCountriesMap.join();
        Map<String, SysDictData> companyTypeMap = companyTypeTask.join();
        Map<String, SysDictData> projectTypeMap = projectTypeTask.join();
        Map<String, SysDictData> requirementMap = requirementTask.join();
        Map<String, SysDictData> productIntentionMap = productIntentionTask.join();
        

        ExcelUtils.UploadResult<LeadInfoHelper> result = ExcelUtils.checkData(leadInfoHelpers, it -> {
            List<String> businessErrors = new ArrayList<>();

            // TODO 抽取 公共校验
            it.setOrigin(Optional.ofNullable(originMap.get(it.getOrigin())).map(SysDictData::getDictValue).orElseGet(() -> {
                // TODO 来源应该可以通过 excel 获取
                if (StringUtils.isNotBlank(it.getOrigin())) {
                    businessErrors.add("来源【" + it.getOrigin() + "】不存在");
                }
                return null;
            }));
            // 区域
            AreaWithCountriesVo vo = areaWithCountriesVoMap.get(it.getAreaName());
            if (vo != null) {
                it.setAreaId(vo.getAreaId());
                // TbCountry country = vo.getCountryList().stream().filter(c -> c.getCountryName().equals(it.getCountryName())).findFirst().orElse(null);
                TbCountry country = vo.getCountryList().stream().filter(c -> countryGetter.apply(c).equals(it.getCountryName())).findFirst().orElse(null);
                if (country != null) {
                    it.setCountryId(country.getCountryId());
                } else {
                    if (StringUtils.isNotBlank(it.getCountryName())) {
                        businessErrors.add("区域【" + it.getAreaName() + "】下不存在国家【" + it.getCountryName() + "】");
                    }
                }
            } else {
                if (StringUtils.isNotBlank(it.getAreaName())) {
                    businessErrors.add("区域【" + it.getAreaName() + "】不存在");
                }
            }

            // it.setSector(Optional.ofNullable(sectorMap.get(it.getSector())).map(SysDictData::getDictValue).orElseGet(() -> {
            //     if (StringUtils.isNotBlank(it.getSector())) {
            //         businessErrors.add("行业【" + it.getSector() + "】不存在");
            //     }
            //     return null;
            // }));
            it.setPriority(Optional.ofNullable(priorityMap.get(it.getPriority())).map(SysDictData::getDictValue).orElseGet(() -> {
                if (StringUtils.isNotBlank(it.getPriority())) {
                    businessErrors.add("优先级【" + it.getPriority() + "】不存在");
                }
                return null;
            }));

            it.setType(Optional.ofNullable(typeMap.get(it.getType())).map(SysDictData::getDictValue).orElseGet(() -> {
                if (StringUtils.isNotBlank(it.getType())) {
                    businessErrors.add("客户类型【" + it.getType() + "】不存在");
                }
                return null;
            }));

            it.setCompanyType(Optional.ofNullable(companyTypeMap.get(it.getCompanyType())).map(SysDictData::getDictValue).orElseGet(() -> {
                if (StringUtils.isNotBlank(it.getCompanyType())) {
                    businessErrors.add("公司类型【" + it.getCompanyType() + "】不存在");
                }
                return null;
            }));

            it.setSegment(Optional.ofNullable(projectTypeMap.get(it.getSegment())).map(SysDictData::getDictValue).orElseGet(() -> {
                if (StringUtils.isNotBlank(it.getSegment())) {
                    businessErrors.add("项目类型【" + it.getSegment() + "】不存在");
                }
                return null;
            }));

            it.setRequirementType(Optional.ofNullable(requirementMap.get(it.getRequirementType())).map(SysDictData::getDictValue).orElseGet(() -> {
                if (StringUtils.isNotBlank(it.getRequirementType())) {
                    businessErrors.add("需求类型【" + it.getRequirementType() + "】不存在");
                }
                return null;
            }));
            it.setIntentionalProduct(Optional.ofNullable(productIntentionMap.get(it.getIntentionalProduct())).map(SysDictData::getDictValue).orElseGet(() -> {
                if (StringUtils.isNotBlank(it.getIntentionalProduct())) {
                    businessErrors.add("产品意向【" + it.getIntentionalProduct() + "】不存在");
                }
                return null;
            }));
            

            // 邮箱和联系方式二选一
            if (StringUtils.isBlank(it.getEmail()) && StringUtils.isBlank(it.getContact())) {
                businessErrors.add("邮箱和联系方式至少填写一项");
            }

            return businessErrors;
        });
        return result;
    }

    /**
     * 异步加载字典数据任务
     *
     * @param dictGetter
     * @param dictType
     * @return
     */
    private CompletableFuture<Map<String, SysDictData>> asyncLoadDictFuture(Function<SysDictData, String> dictGetter, String dictType) {
        return CompletableFuture.supplyAsync(() -> getLocalLabel2DictMap(dictGetter, dictType), threadPoolTaskExecutor);
    }

    private Map<String, SysDictData> getLocalLabel2DictMap(Function<SysDictData, String> dictGetter, String type) {
        List<SysDictData> priorities = sysDictDataMapper.selectDictDataByType(type);
        return priorities.stream().collect(Collectors.toMap(dictGetter, dict -> dict));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public List<String> insertLeadImportExcel(BasicTableInfoHelper basicTableInfoHelper, List<LeadInfoHelper> transform) {
        // TODO 手动事务

        List<TcLead> tcLeads = transform.stream().map(helper -> {
            TcLead tcLead = BeanUtil.copyProperties(helper, TcLead.class);
            tcLead.setCreateBy(SecurityUtils.getUserId() + "");
            tcLead.setCreateTime(new Date());
            // Optional.ofNullable(tcLead.getLeadTime()).ifPresent(leadTime -> {
            //     tcLead.setLeadTime(DateUtils.CNTimeDate(leadTime, SecurityUtils.getTimeZone()));
            // });
            tcLead.setStatus("1");
            tcLead.setPermissionChar(SecurityUtils.getPermissionCharIns());

            // 如果创建线索用户是区域:全球，所属：总部；即表示是市场部创建的线索，所属是空。 其他情况，所属：自己
            if (SecurityUtils.getDeptId() == 0L && SecurityUtils.getLoginUser().getAreaId() == 0L) {
                tcLead.setOrigin("1");
            } else {
                tcLead.setOwner(SecurityUtils.getUserId());
                // 如果是代理商创建的线索，所属代理商是自己，并且分配给自己
                if (SecurityUtils.getDeptId() != 0L) {
                    tcLead.setOwnerAgent(SecurityUtils.getDeptId());
                    tcLead.setStatus("2");
                }
            }
            String leadNo = redisCache.generateSerialNumber(LEAF.getPrefix());
            tcLead.setLeadNo(leadNo);
            return tcLead;
        }).collect(Collectors.toList());

        // baseMapper.batchInsert(tcLeads);
        boolean b = saveBatch(tcLeads, 100);


        return Collections.emptyList();

    }

    @Override
    public List<TcLead> listByCurUser(LeadForAccountReq forAccountReq) {
        forAccountReq.setOwner(SecurityUtils.getUserId());
        forAccountReq.setPermissionChar(SecurityUtils.getPermissionCharSea());
        return baseMapper.listByCurUser(forAccountReq);
    }
    @Override
    public List<LeadRes> curCountry() {
        return baseMapper.curCountry(SecurityUtils.getPermissionCharSea(), LocaleContextHolder.getLocale().getCountry());
    }


}
