package com.ruoyi.project.business.service.task;


import cn.hutool.core.collection.CollectionUtil;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.project.business.domain.TbArea;
import com.ruoyi.project.business.domain.TcLead;
import com.ruoyi.project.business.domain.TcLeadHistory;
import com.ruoyi.project.business.service.ITbAreaService;
import com.ruoyi.project.business.service.ITcLeadHistoryService;
import com.ruoyi.project.business.service.ITcLeadService;
import com.ruoyi.project.system.domain.SysUser;
import com.ruoyi.project.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.support.TransactionTemplate;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author fangk
 * @date 2024/4/24
 * @desc LeadTaskService 线索任务 service
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class LeadTaskService {

    private final ITcLeadService tcLeadService;
    private final ISysUserService sysUserService;
    private final ITbAreaService tbAreaService;
    private final TransactionTemplate transactionTemplate;
    private final ITcLeadHistoryService tcLeadHistoryService;


    public void autoAssignLead() {

        /* 
        查询全部来自市场部并且未分配的线索:
         目前逻辑是，线索新增时，当新增用户的所属是总部，并且区域是全球时， 线索表的 owner 不设值,表示市场部的线索。
         否则设为当前用户的id，表示是用户的线索。
         */
        List<TcLead> tcLeads = tcLeadService.selectLeadsFromMarketAndUnassigned();
        if (CollectionUtil.isEmpty(tcLeads)) {
            log.info("市场部线索自动分配任务结束，没有需要分配的线索");
            return;
        }

        // 线索无法分配到全球，也就是至少在大区，权限字符至少有两个。此时倒数第二个一定是大区的权限字符即大区id
        List<Long> areaIds = tcLeads.stream().map(TcLead::getAreaId).collect(Collectors.toList());
        List<TbArea> areas = tbAreaService.selectAreaListByIds(areaIds);
        //List<String> areaList = areas.stream()
        //        .map(area -> area.getPermissionChar().split(","))
        //        .flatMap(Arrays::stream)
        //        .collect(Collectors.toList());
        // 去除大区权限字符，收集成：areaId -> 大区areaId；即权限字符的第一个是当前区域id为key， 权限字符的最后一个是当前区域对应的大区id为value
        //Map<Long, Long> areaMap = areas.stream()
        //        .collect(Collectors.toMap(
        //                        TbArea::getAreaId, 
        //                        area -> Long.parseLong(area.getPermissionChar().split(",")[area.getPermissionChar().split(",").length - 1])));
        //Map<Long, Long> curArea2BigAreaMap = areas.stream()
        //        .map(TbArea::getPermissionChar)
        //        .map(permissionChar -> permissionChar.split(","))
        //        //.flatMap(Arrays::stream)
        //        //.collect(Collectors.toMap(permissionChar -> p
        //        .collect(Collectors.toMap(item -> Long.parseLong(item[0]), item -> Long.parseLong(item[item.length - 1])));
        Map<Long, Long> curArea2BigAreaMap = areas.stream().map(TbArea::getPermissionChar)
                // 左边第二个字符：key； 右边第4个字符：value
                .collect(Collectors.toMap(item -> Long.parseLong(item.charAt(1) + ""), item -> Long.parseLong(item.charAt(item.length() - 4) + ""), (k1, k2) -> k2));


        for (TcLead lead : tcLeads) {
            // 根据线索所属区域自动分配给对应的区总
            Long areaId = lead.getAreaId();
            // 获取大区id
            Long bigAreaId = curArea2BigAreaMap.get(areaId);
            List<SysUser> sysUsers = sysUserService.selectUserByRoleKeyAreaIdAgentId("area_sales_director", bigAreaId, null, null, null);
            // 大区区总只有一个
            if (CollectionUtil.isNotEmpty(sysUsers)) {
                if (sysUsers.size() > 1) {
                    log.error("当前线索：【{}】所属区域：【{}】存在多个区总:【{}】", lead.getLeadNo(), areaId, sysUsers.stream().map(SysUser::getUserName).collect(Collectors.joining(",")));
                }
                SysUser sysUser = sysUsers.get(0);

                transactionTemplate.execute(status -> {
                    try {
                        // 跟新线索
                        boolean r1 = updateLead(lead, sysUser);

                        // 插入履历
                        boolean r2 = insertLeadHis(lead, sysUser);
                        //return r1 && r2;
                        if (r1 && r2) {
                            return true;
                        } else {
                            throw new RuntimeException("线索：【" + lead.getLeadNo() + "】分配给区总：【" + sysUser.getUserName() + "】失败");
                        }
                    } catch (Exception e) {
                        // TODO 保存失败数据，发送站内信？？
                        status.setRollbackOnly();
                        log.error("线索：【{}】分配给区总：【{}】失败", lead.getLeadNo(), sysUser.getUserName(), e);
                    }
                    return false;
                });

                // TODO 履历
            } else {
                // i18n
                log.error("当前线索：【{}】所属区域：【{}】不存在区总", lead.getLeadNo(), areaId);
            }
        }
    }


    private boolean updateLead(TcLead lead, SysUser sysUser) {

        // 2： 已分配
        lead.setStatus("2");
        lead.setOwner(sysUser.getUserId());
        //lead.setUpdateBy(sysUser.getUserName());
        //lead.setUpdateTime(DateUtils.getNowDate());
        lead.setPermissionChar(sysUser.getPermissionChar());
        // TODO i18n
        lead.setRemark("市场部线索自动分配任务");
        //return tcLeadService.updateTcLead(lead);
        return tcLeadService.updateById(lead);
    }

    private boolean insertLeadHis(TcLead lead, SysUser sysUser) {
        
        // 使用 admin作为操作人 这里是没法获取到操作人，所以手动赋值了
        TcLeadHistory history = new TcLeadHistory(
                lead.getLeadId(), 
                1L,
                1L, 
                null,
                null,
                sysUser.getUserId(),
                1L,
                DateUtils.getNowDate(),
                // TODO i18n 
                "市场部线索自动分配任务");
        //return tcLeadHistoryService.insertTcLeadHistory(history);
        return tcLeadHistoryService.save(history);
    }
}